using System;
using System.Windows.Forms;

using wclIrDA;
using wclCommon;

namespace IrDADiscovery
{
    public partial class fmMain : Form
    {
        private wclIrDAReceiver wclIrDAReceiver;
        private wclPowerEventsMonitor FMonitor;

        public fmMain()
        {
            InitializeComponent();
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            wclIrDAReceiver = new wclIrDAReceiver();
            wclIrDAReceiver.OnChange += new EventHandler(wclIrDAReceiver_OnChange);
            wclIrDAReceiver.OnDevicesFound += new wclIrDADevicesFoundEvent(wclIrDAReceiver_OnDevicesFound);

            cbClassName.SelectedIndex = 0;
            cbClassName_SelectedIndexChanged(cbClassName, EventArgs.Empty);

            FMonitor = new wclPowerEventsMonitor();
            FMonitor.OnPowerStateChanged += new wclPowerStateChangedEvent(FMonitor_OnPowerStateChanged);
            FMonitor.Open();
        }

        void FMonitor_OnPowerStateChanged(object Sender, wclPowerState State)
        {
            switch (State)
            {
                case wclPowerState.psResumeAutomatic:
                    lbEvents.Items.Add("Power state: psResumeAutomatic");
                    break;

                case wclPowerState.psResume:
                    lbEvents.Items.Add("Power state: psResume");
                    break;

                case wclPowerState.psSuspend:
                    wclIrDAReceiver.StopLazyDiscovering();
                    wclIrDAReceiver.Unsubscribe();

                    lbEvents.Items.Add("Power state: psSuspend");
                    break;

                case wclPowerState.psUnknown:
                    lbEvents.Items.Add("Power state: psUnknown");
                    break;
            }
        }

        void wclIrDAReceiver_OnDevicesFound(object Sender, wclIrDADevice Device)
        {
            lbEvents.Items.Add("Device found: " + Device.Address.ToString("X8") + "; " + Device.Name);
        }

        void wclIrDAReceiver_OnChange(object sender, EventArgs e)
        {
            if (wclIrDAReceiver.Available)
                lbEvents.Items.Add("Hardware changed. IrDA available.");
            else
                lbEvents.Items.Add("Hardware changed. IrDA NOT available.");
        }

        private void cbClassName_SelectedIndexChanged(object sender, EventArgs e)
        {
            cbAttributeName.Items.Clear();
            
            switch (cbClassName.SelectedIndex)
            {
                case 0:
                    cbAttributeName.Items.Add("DeviceName");
                    cbAttributeName.Items.Add("IrLMPSupport");
                    break;
                case 1:
                    cbAttributeName.Items.Add("Parameters");
                    cbAttributeName.Items.Add("IrDA:IrLMP:LsapSel");
                    cbAttributeName.Items.Add("IrDA:TinyTP:LsapSel");
                    cbAttributeName.Items.Add("IrDA:InstanceName");
                    break;
                case 2:
                    cbAttributeName.Items.Add("IrDA:TinyTP:LsapSel");
                    break;
                case 3:
                    cbAttributeName.Items.Add("IrDA:IrLMP:LsapSel");
                    break;
                case 4:
                    cbAttributeName.Items.Add("Integer value");
                    cbAttributeName.Items.Add("Octet Sequence");
                    cbAttributeName.Items.Add("User string");
                    break;
            }
            
            cbAttributeName.SelectedIndex = 0;
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            wclIrDAReceiver.StopLazyDiscovering();
            wclIrDAReceiver.Unsubscribe();

            FMonitor.Close();
        }

        private void btDiscover_Click(object sender, EventArgs e)
        {
            lvDevices.Items.Clear();
  
            wclIrDADevice[] Devices = null;
            Int32 Res = wclIrDAReceiver.Discover(out Devices);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            else
                if (Devices == null || Devices.Length == 0)
                    MessageBox.Show("Nothing found");
                else
                    for (Int32 i = 0; i < Devices.Length; i++)
                    {
                        wclIrDADevice Device = Devices[i];

                        ListViewItem Item = lvDevices.Items.Add(Device.Address.ToString("X8"));
                        Item.SubItems.Add(Device.Name);
                        Item.SubItems.Add(Device.Hints.ToString());
                        Item.SubItems.Add(Device.CharSet.ToString());
                    }
        }

        private void btSubscribe_Click(object sender, EventArgs e)
        {
            Int32 Res = wclIrDAReceiver.Subscribe();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btUnsubscribe_Click(object sender, EventArgs e)
        {
            Int32 Res = wclIrDAReceiver.Unsubscribe();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btClear_Click(object sender, EventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btStartLazyDiscovering_Click(object sender, EventArgs e)
        {
            Int32 Res = wclIrDAReceiver.StartLazyDiscovering();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btStopLazyDiscovering_Click(object sender, EventArgs e)
        {
            Int32 Res = wclIrDAReceiver.StopLazyDiscovering();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btQuery_Click(object sender, EventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                wclIrDAAttribute Attrib;
                Int32 Res = wclIrDAReceiver.QueryAttribute(Convert.ToUInt32(lvDevices.SelectedItems[0].Text, 16),
                    cbClassName.SelectedItem.ToString(), cbAttributeName.SelectedItem.ToString(),
                    out Attrib);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                else
                {
                    switch (Attrib.ValueType)
                    {
                        case wclIrDAAttributeType.atInteger:
                            MessageBox.Show("Integer: " + Attrib.IntegerValue.ToString());
                            break;

                        case wclIrDAAttributeType.atSequence:
                            String Str = "";
                            if (Attrib.SequenceValue != null)
                            {
                                if (Attrib.SequenceValue.Length > 0)
                                    for (int i = 0; i < Attrib.SequenceValue.Length; i++)
                                        Str = Str + Attrib.SequenceValue[i].ToString("X2");

                                Attrib.SequenceValue = null;
                            }

                            MessageBox.Show("Sequence: " + Str);
                            break;

                        case wclIrDAAttributeType.atString:
                            MessageBox.Show("String: " + Attrib.StringValue);
                            break;
                    }
                }
            }
        }
    }
}